const inquirer = require('inquirer');
const { ncp } = require('ncp');
const path = require('path');
const editJsonFile = require('edit-json-file');
const { existsSync } = require('fs');

const { frameWorkConfig } = require('./framework.config');
const { getTemplateDir } = require('./util-files');
const dir = path.join(__dirname, '..', '..', 'templates');

/**
 * @method chooseTemplates
 * @description Choose a template.
 */
const chooseTemplates = async () => {
  const directories = await getTemplateDir();
  const { chooseTemplates } = await inquirer.prompt([
    {
      type: 'list',
      name: 'chooseTemplates',
      message: 'Select a template',
      choices: [...directories, new inquirer.Separator()],
    },
  ]);

  return chooseTemplates;
};

const validateFramework = (framework) => {
  const config = frameWorkConfig[framework];
  if (!config)
    throw new Error(
      'Sorry but this template doesnt have any configuration yet :('
    );

  const source = path.join(dir, framework, 'default');
  const isExist = existsSync(path.join(source));
  if (!isExist)
    throw new Error(
      'Sorry but this template doesnt have any configuration yet :('
    );
};

const chooseToolsToWork = async (framework, spinner, destination) => {
  const wasGeneratedDefaultFrameworkTemplate = existsSync(
    path.join(destination)
  );
  if (!wasGeneratedDefaultFrameworkTemplate) {
    throw new Error('Sorry but this template does not exists :(');
  }

  if (!framework)
    throw new Error('Sorry but this template is not configured yet :(');

  const config = frameWorkConfig[framework];
  if (!config)
    throw new Error(
      'Sorry but this template doesnt have any configuration yet :('
    );

  if (!config.tools) return;
  const tools = Object.entries(config.tools);
  if (!tools.length) return;

  for (const [tool, optionsObj] of tools) {
    const options = optionsObj.map(({ key }) => key);
    const { chooseOptionsFromTool } = await inquirer.prompt([
      {
        type: 'list',
        name: 'chooseOptionsFromTool',
        message: 'Select an option for ' + tool,
        choices: [...options, new inquirer.Separator()],
      },
    ]);

    const optionSelected = optionsObj.find(
      ({ key }) => key === chooseOptionsFromTool
    );
    if (!optionSelected.install) {
      spinner.text = 'Sorry but this option has not installation yet :(';
      continue;
    }

    const optionDir = path.join(dir, framework, chooseOptionsFromTool);
    const isExist = existsSync(optionDir);

    if (!isExist) {
      spinner.text = 'Sorry but this option is not configured yet :(';
      continue;
    }

    spinner.text = 'Adding this option to your template :)';
    await optionSelected.install(destination, dir, spinner);
  }
};

/**
 * @method copyProjectFiles
 * @description Duplicate the template.
 */
const copyProjectFiles = async (destination, framework) => {
  return new Promise((resolve, reject) => {
    const source = path.join(dir, framework, 'default');
    const options = {
      clobber: true,
      stopOnErr: true,
    };

    ncp.limit = 16;
    ncp(source, destination, options, function (err) {
      if (err) reject(err);
      resolve();
    });
  });
};

/**
 * @method updatePackageJson
 * @description Edit package.json.
 */
const updatePackageJson = async (destination) => {
  const file = editJsonFile(`${destination}/package.json`, { autosave: true });

  file.set('name', path.basename(destination));
};

module.exports = {
  chooseTemplates,
  copyProjectFiles,
  updatePackageJson,
  chooseToolsToWork,
  validateFramework,
};
